import 'dart:math';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'package:gameolive/gameolive.dart';
import 'package:gameolive/models/config.dart';
import 'package:gameolive/models/game.dart';
import 'package:gameolive/models/gamesResponse.dart';
import 'package:gameolive/models/launchConfig.dart';
import 'package:gameolive/models/playerBalance.dart';

import '/components/bottom_bar.dart';
import 'GameOliveView.dart';
import 'components/left_bar.dart';
import 'components/right_bar.dart';
import 'components/top_bar.dart';

const clientId =
    'dashboard-client-bdb2c3b0-98f5-4a27-8223-2bf1448c7299@739821cf-b7ac-4266-a037-02823910e0e9.gol';
// "fb_game_sa-4862d2b3-fb68-4963-b47b-beec6af422a5@05cc351f-dbd0-43af-aaaf-515ffaecdd34.gol";
const clientSecret = 'gol1658046241679'; //"abc1602937927739";
const operatorId = '62d3c640491257000b394769'; //"5f8ae3cbc34272000af1f3bf";
const server = 'https://elantra-api.gameolive.com';
const static = 'https://static.luckybeetlegames.com';
const walletClientId =
    "wallet_manager-6f18a5bb-9b2e-4ed8-b7db-abf6465256f8@4e539d5f-4dd9-4518-8e67-49e401ea0b4b.gol";
const walletClientSecret = "gol1606078109162";

void main() {
  runApp(const MyApp());
}

class MyApp extends StatelessWidget {
  const MyApp({Key? key}) : super(key: key);

  // This widget is the root of your application.
  @override
  Widget build(BuildContext context) {
    SystemChrome.setEnabledSystemUIMode(SystemUiMode.immersiveSticky);

    return MaterialApp(
      title: 'Flutter Demo',
      theme: ThemeData(
        primarySwatch: Colors.blue,
        textTheme: Theme.of(context).textTheme.apply(
              bodyColor: Colors.white,
              displayColor: Colors.white,
            ),
      ),
      home: const MyHomePage(title: 'Flutter Demo Home Page'),
    );
  }
}

class MyHomePage extends StatefulWidget {
  const MyHomePage({Key? key, required this.title}) : super(key: key);

  final String title;

  @override
  State<MyHomePage> createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {
  final _scaffoldKey = GlobalKey<ScaffoldState>();

  String _platformVersion = 'Unknown';
  final _gameolivePlugin = Gameolive();

  List<Game>? _games;
  LaunchConfig? inlineLaunchConfig;
  PlayerBalance? _playerBalance;
  String defaultPlayerId = "";
  final String _playerId = "";
  final String _playerToken = "";

  @override
  void initState() {
    super.initState();
    // if (widget.gameLaunchConfig!.orientation == "landscape") {
    SystemChrome.setPreferredOrientations([
      DeviceOrientation.landscapeRight,
      DeviceOrientation.landscapeLeft,
    ]);
    // }
    SystemChrome.setEnabledSystemUIMode(SystemUiMode.manual, overlays: []);

    initPlatformState();
    initGameOlive();
  }

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initPlatformState() async {
    String platformVersion;
    // Platform messages may fail, so we use a try/catch PlatformException.
    // We also handle the message potentially returning null.
    try {
      platformVersion = await _gameolivePlugin.getPlatformVersion() ??
          'Unknown platform version';
    } on PlatformException {
      platformVersion = 'Failed to get platform version.';
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    setState(() {
      _platformVersion = platformVersion;
    });
  }

  initGameOlive() async {
    Config config = Config(
        operatorId: operatorId,
        clientId: clientId,
        clientSecret: clientSecret,
        server: server,
        static: static);
    List<Game>? games;
    try {
      await _gameolivePlugin.init(config); // initialize the library

      GamesResponse gamesResponse =
          await _gameolivePlugin.getGames(10, 0); // get first 10 games
      games = gamesResponse.games;
    } on PlatformException {
      // Log exception and report studio@gameolive.com
    }

    setState(() {
      _games = games;
    });
  }

  // @override
  // dispose() {
  //   // SystemChrome.setPreferredOrientations([
  //   //   DeviceOrientation.landscapeRight,
  //   //   DeviceOrientation.landscapeLeft,
  //   //   DeviceOrientation.portraitUp,
  //   //   DeviceOrientation.portraitDown,
  //   // ]);
  //   // SystemChrome.setEnabledSystemUIMode(SystemUiMode.manual,
  //   //     overlays: SystemUiOverlay.values);

  //   super.dispose();
  // }

  @override
  Widget build(BuildContext context) {
    return Container(
        decoration: const BoxDecoration(
            // image: DecorationImage(
            //   image: AssetImage("assets/lobby/base_background.png"),
            //   fit: BoxFit.cover,
            // ),
            ),
        child: SafeArea(
            top: false,
            left: false,
            bottom: false,
            right: false,
            child: Scaffold(
                backgroundColor: Colors.transparent,
                body: LayoutBuilder(
                  builder: (BuildContext context, BoxConstraints constraints) {
                    Size referenceSize = const Size(1280, 720);
                    var ratio = min(constraints.maxWidth / referenceSize.width,
                        constraints.maxHeight / referenceSize.height);

                    return Align(
                      alignment: Alignment.topCenter,
                      child: Container(
                        // color: Colors.amber,
                        decoration: const BoxDecoration(
                          color: Colors.transparent,
                          image: DecorationImage(
                            image:
                                AssetImage("assets/lobby/base_background.png"),
                            fit: BoxFit.cover,
                          ),
                        ),
                        width: referenceSize.width * ratio,
                        height: referenceSize.height * ratio,
                        child: Column(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: [
                            TopBar(referenceSize: referenceSize, ratio: ratio),
                            //xSizedBox(height: 20 * ratio),
                            Row(
                              mainAxisAlignment: MainAxisAlignment.spaceBetween,
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: [
                                LeftBar(
                                    referenceSize: referenceSize, ratio: ratio),
                                SizedBox(
                                    width: 980 * ratio,
                                    height: 500 * ratio,
                                    child: _games == null
                                        ? const Text('Execution is in progress')
                                        : _games!.isEmpty
                                            ? const Text(
                                                'There are no games in the offerings for this platform')
                                            : ListView.builder(
                                                itemCount: _games!.length,
                                                // This next line does the trick.
                                                scrollDirection:
                                                    Axis.horizontal,
                                                itemBuilder: (context, index) {
                                                  final item = _games![index];
                                                  return Container(
                                                    padding: EdgeInsets.all(
                                                        10 * ratio),
                                                    width: 360.0 * ratio,
                                                    child: GestureDetector(
                                                      onTap: () {
                                                        LaunchConfig
                                                            launchConfig =
                                                            LaunchConfig();
                                                        launchConfig.server =
                                                            server;
                                                        launchConfig.static =
                                                            static;
                                                        launchConfig
                                                                .operatorId =
                                                            operatorId;
                                                        launchConfig.configId =
                                                            item.configuration;
                                                        launchConfig
                                                                .orientation =
                                                            "landscape";
                                                        launchConfig.playerId =
                                                            _playerId;
                                                        launchConfig
                                                                .playerToken =
                                                            _playerToken;
                                                        Navigator.push(
                                                            context,
                                                            MaterialPageRoute(
                                                              builder: (context) =>
                                                                  GameView(
                                                                      instance:
                                                                          _gameolivePlugin,
                                                                      launchConfig:
                                                                          launchConfig,
                                                                      onGoToHome:
                                                                          (value) {
                                                                        Navigator.pop(
                                                                            context);
                                                                      }),
                                                            ));
                                                      },
                                                      child:
                                                          // Image.network(
                                                          //     item.photos != null &&
                                                          //             item.photos!
                                                          //                 .isNotEmpty
                                                          //         ? item.photos![0]
                                                          //                 .downloadUrl ??
                                                          //             ""
                                                          //         : "")
                                                          Image.asset(
                                                        "assets/games/game$index.png",
                                                        fit: BoxFit.cover,
                                                      ),
                                                    ),
                                                  );
                                                }
                                                // children: <Widget>[
                                                //   Container(
                                                //     padding: EdgeInsets.all(10 * ratio),
                                                //     width: 360.0 * ratio,
                                                //     child: Image.asset(
                                                //       "assets/games/game1.png",
                                                //       fit: BoxFit.cover,
                                                //     ),
                                                //   ),
                                                //   Container(
                                                //     padding: EdgeInsets.all(10 * ratio),
                                                //     width: 360.0 * ratio,
                                                //     child: Image.asset(
                                                //       "assets/games/game2.png",
                                                //       fit: BoxFit.cover,
                                                //     ),
                                                //   ),
                                                //   Container(
                                                //     padding: EdgeInsets.all(10 * ratio),
                                                //     width: 360.0 * ratio,
                                                //     child: Image.asset(
                                                //       "assets/games/game3.png",
                                                //       fit: BoxFit.cover,
                                                //     ),
                                                //   ),
                                                //   Container(
                                                //     padding: EdgeInsets.all(10 * ratio),
                                                //     width: 360.0 * ratio,
                                                //     child: Image.asset(
                                                //       "assets/games/game4.png",
                                                //       fit: BoxFit.cover,
                                                //     ),
                                                //   ),
                                                //   Container(
                                                //     padding: EdgeInsets.all(10 * ratio),
                                                //     width: 360.0 * ratio,
                                                //     child: Image.asset(
                                                //       "assets/games/game4.png",
                                                //       fit: BoxFit.cover,
                                                //     ),
                                                //   ),
                                                // ],
                                                )),
                                RightBar(
                                    referenceSize: referenceSize, ratio: ratio),
                              ],
                            ),
                            BottomBar(
                                referenceSize: referenceSize, ratio: ratio),
                          ],
                        ),
                      ),
                    );
                  },
                ))));
  }
}
